var scriptName = "Curve onto BG v0.1.1";

@version 2.4
@warnings
@strict
@script modeler

var pol;
var nTri;
var lifting = 0;
var backCurve = false;
var offset = 0;

main
{
    return if !Panel();
    selmode(USER);
    var fg = lyrfg();
    var bg = lyrbg();
    if(bg == nil) return;
    editbegin();
    var nCurve = 0;
    var mc = 0;
    var poly;
    var curves[1];
    foreach(poly,polygons){
        if(poly.isCurve() == false) continue;
        nCurve++;
        curves[nCurve] = poly;
        var pi = polyinfo(poly);
        mc += size(pi)- 1;
    }
    editend();
    if(mc <= 0) return;
    lyrsetfg(bg);
    triple();
    selpolygon(SET,NVEQ,3);
    nTri = polycount(); nTri = nTri[4];
    if(nTri == 0){
        undo();
        lyrsetfg(fg);
        lyrsetbg(bg);
        return;
    }
    pol = array(nTri,10);
    moninit((mc+1)*nTri);
    var abort = GetBGInfo();
    undo();
    lyrsetfg(fg);
    lyrsetbg(bg);
    if(abort) return;
    var newCurves[1], nci = 0;
    selpolygon(SET,POLYID,curves);
    if(backCurve){
        copy();
        fg = lyrempty();
        fg.sortA();
        lyrsetfg(fg[1]);
        paste();
    }
    editbegin();
    var poly;
    foreach(poly,polygons){
        if(poly.isCurve() == false) continue;
        var pi = polyinfo(poly);
        var j, np = size(pi);
        var bc[np- 1];
        for(j = 1;j < np;j++){
            var p = pi[j+1];
            var pp = pointinfo(p);
            var minD = 1.0e+38, dp = nil, dn;
            var i;
            for(i = 1;i <= nTri;i++){
                if(pol[i,4] != nil){
                    var near;
                    near = NearestPoint(pol[i],pp);
                    var d = vmag(pp-near);
                    if(d < minD){
                        minD = d;
                        dp = near;
                        dn = pol[i,4];
                    }else if(dp == near) {
                        dn += pol[i,4];
                    }
                }
                if(monstep()){
                    monend();
                    editend(ABORT);
                    return;
                }
            }
            if(dp != nil){
                dn /= vmag(dn);
                pointmove(p,dp+dn*(offset+lifting));
                if(backCurve){
                    bc[j] = addpoint(dp+dn*(offset-lifting));
                }
            }
        }
        if(backCurve){
            nci++;
            newCurves[nci] = poly.hasCCStart()?
                addcurve(bc,poly.surface,START):
                addcurve(bc,poly.surface);
            if(!poly.hasCCEnd()) newCurves[nci] = nil; 
        }
    }
    monend();
    editend();
    if(size(newCurves) > 0){
        selpolygon(CLEAR);
        selpolygon(SET,POLYID,newCurves);
        toggleCCend();
    }
    selpolygon(CLEAR);
}

NearestPoint: pi,pp
{
    var n = pi[4];
    var p1 = pi[1];
    var p0 = pp-n*dot3d(pp-p1,n);
    var v = p0-p1;
    if(dot3d(v,pi[5])<0){
        var d = min(1,max(0,dot3d(v,pi[6])));
        return p1*(1-d)+pi[2]*d;
    }
    if(dot3d(v,pi[7])<0){
        var d = min(1,max(0,dot3d(v,pi[8])));
        return p1*(1-d)+pi[3]*d;
    }
    v = p0-pi[2];
    if(dot3d(v,pi[9])<0){
        var d = min(1,max(0,dot3d(v,pi[10])));
        return pi[2]*(1-d)+pi[3]*d;
    }
    return p0;
}

GetBGInfo
{
    editbegin();
    var i;
    for(i = 1;i <= nTri;i++){
        var pi = polyinfo(polygons[i]);
        var n = polynormal(polygons[i]);
        pol[i,4] = n;
        if(n == nil) continue;
        pol[i,1] = pointinfo(pi[2]);
        pol[i,2] = pointinfo(pi[3]);
        pol[i,3] = pointinfo(pi[4]);
        var v;
        v = pol[i,2]-pol[i,1];
        pol[i,5] = cross3d(n,v);
        pol[i,6] = v/dot3d(v,v);
        v = pol[i,3]-pol[i,1];
        pol[i,7] = cross3d(v,n);
        pol[i,8] = v/dot3d(v,v);
        v = pol[i,3]-pol[i,2];
        pol[i,9] = cross3d(n,v);
        pol[i,10] = v/dot3d(v,v);
        if(monstep()){
            editend(ABORT);
            return true;
        }
    }
    editend();
    return false;
}

Panel
{
    reqbegin(scriptName);
    var c1 = ctldistance("  Lifting",lifting);
    var c2 = ctlcheckbox("Create Back Curve",backCurve);
    var c3 = ctldistance("  Lifting Offset",offset);
    ctlactive(c2,"CABackCurve",c3);
    return false if !reqpost();
    lifting = getvalue(c1);
    backCurve = lifting != 0 && getvalue(c2);
    offset = backCurve? getvalue(c3): 0;
    reqend();
    return true;
}

CABackCurve:f
{
    return f;
}
